@@ -1,6 +1,6 @@
 /* about.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2011-2023 Ralf Hoffmann.
+ * Copyright (C) 2011-2025 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -92,7 +92,7 @@ void About::about()
     ac1_texts->setMaxSpace( 5 );
     ac1_texts->setBorderWidth( 5 );
 
-    std::string str1 = AGUIXUtils::formatStringToString( "Worker %d.%d.%d%s|Copyright (C) 1998 - 2023 Ralf Hoffmann||Active options:",
+    std::string str1 = AGUIXUtils::formatStringToString( "Worker %d.%d.%d%s|Copyright (C) 1998 - 2025 Ralf Hoffmann||Active options:",
                                                          WORKER_MAJOR, WORKER_MINOR, WORKER_PATCH, WORKER_COMMENT );
     win->addMultiLineText( str1, *ac1_texts, 0, 0, NULL, NULL );
 
@@ -347,7 +347,7 @@ void About::showLicense( AWindow *transient_window )
     if ( ! m_aguix ) return;
 
     RefCount<AFontWidth> lencalc( new AFontWidth( m_aguix, NULL ) );
-    TextStorageString ts( cgpl, lencalc );
+    auto ts = std::make_shared< TextStorageString >( cgpl, lencalc );
     Worker::getRequester()->request( catalog.getLocale( 509 ), ts, catalog.getLocale( 11 ),
                                      transient_window, Requester::REQUEST_NONE );
 }
