@@ -1,6 +1,6 @@
 /* kartei.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2004-2014 Ralf Hoffmann.
+ * Copyright (C) 2004-2019 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -37,6 +37,8 @@ public:
   Kartei( const Kartei &other );
   Kartei &operator=( const Kartei &other );
   void setOption( AWindow *win, unsigned int option, const char *optionname );
+  void updateOption( unsigned int option, const char *optionname );
+  void updateOption( unsigned int option, const std::string &optionname );
   void maximize();
   static void optionChangeCallback( Kartei *k1, unsigned int option );
   void optionChange( unsigned int option );
