@@ -57,8 +57,8 @@ public:
   bool isPosChangeForbidden() const;
   void setForbidPosChange(bool nv);
   void setStrongKeyCapture(bool nv);
-  virtual void paste(unsigned char*);
-  virtual void cancelpaste();
+  void paste( const unsigned char*) override;
+  void cancelpaste() override;
   virtual void cancelcut();
   void insertAtCursor( const char *str );
   void selectAll( bool copy_selection = true );
