@@ -1,6 +1,6 @@
 /* textstorage.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2005-2022 Ralf Hoffmann.
+ * Copyright (C) 2005-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -69,7 +69,7 @@ int TextStorageString::getNrOfLines() const
 
 int TextStorageString::getLine( int line_nr, unsigned int offset, int len, std::string &return_line ) const
 {
-  unsigned int start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
+  size_t start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
 
   if ( ( line_nr < 0 ) || ( line_nr >= getNrOfLines() ) ) return 1;
 
@@ -107,7 +107,7 @@ int TextStorageString::getLine( int line_nr, unsigned int offset, std::string &r
 
 void TextStorageString::initBuffer( int line_limit_width )
 {
-    unsigned int i;
+    size_t i;
     int current_line = 0, current_subline = 0;
 
     line_offsets.clear();
@@ -219,7 +219,7 @@ int TextStorageString::getMaxLineWidth() const
 
 int TextStorageString::getLineWidth( int line_nr ) const
 {
-  unsigned int start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
+  size_t start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
   int width = -1;
 
   if ( ( line_nr < 0 ) || ( line_nr >= getNrOfLines() ) ) return -1;
@@ -292,7 +292,7 @@ int TextStorageString::getLineLimit()
 
 int TextStorageString::getLineCharacters( int line_nr ) const
 {
-  unsigned int start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
+  size_t start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
 
   if ( ( line_nr < 0 ) || ( line_nr >= getNrOfLines() ) ) return -1;
 
@@ -357,7 +357,7 @@ int TextStorageString::getUnwrappedLine( int line_nr,
                                          std::string &return_line,
                                          bool raw ) const
 {
-    unsigned int start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
+    size_t start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
     
     if ( line_nr < 0 || line_nr >= getNrOfUnwrappedLines() ) return 1;
     
@@ -384,15 +384,10 @@ int TextStorageString::getUnwrappedLine( int line_nr,
         }
 
         if ( m_convert != NO_CHARACTER_CONVERSION && raw ) {
-            if ( start_offset < m_converted_pos_to_raw.size() && line_len > 0 ) {
-                size_t raw_start = m_converted_pos_to_raw.at( start_offset );
+            if ( line_len > 0 ) {
+                size_t raw_start = getRawPosition( start_offset );
 
-                size_t raw_end = start_offset + line_len;
-
-                if ( raw_end >= m_converted_pos_to_raw.size() ) {
-                    raw_end = m_converted_pos_to_raw.size() - 1;
-                }
-                raw_end = m_converted_pos_to_raw.at( raw_end );
+                size_t raw_end = getRawPosition( start_offset + line_len );
 
                 return_line.append( m_raw_buffer, raw_start, raw_end - raw_start );
             }
@@ -441,7 +436,7 @@ int TextStorageString::getUnwrappedLineRaw( int line_nr,
 
 int TextStorageString::getUnwrappedLineCharacters( int line_nr ) const
 {
-    unsigned int start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
+    size_t start_offset, line_len;  // remember offset from line_offsets are not necessarly inside buffer
     
     if ( line_nr < 0 || line_nr >= getNrOfUnwrappedLines() ) return -1;
     
@@ -528,64 +523,27 @@ void TextStorageString::convertRawData()
 
     size_t len = m_raw_buffer.length();
     size_t pos = 0;
-    size_t cl;
     const char *cstr = m_raw_buffer.c_str();
 
     buffer = "";
-    m_converted_pos_to_raw.clear();
+    m_converted_pos_to_raw_snapshots.clear();
 
-    size_t column = 0;
+    uint8_t column = 0;
 
     while ( pos < len ) {
-        if ( cstr[pos] >= 0 && cstr[pos] <= 127 ) {
-            if ( std::isprint( cstr[pos] ) ) {
-                m_converted_pos_to_raw.push_back( pos );
-                buffer += cstr[pos++];
-                column++;
-            } else if ( cstr[pos] == '\n' ) {
-                m_converted_pos_to_raw.push_back( pos );
-                buffer += cstr[pos++];
-                column = 0;
-            } else {
-                std::string replacement = getReplacementString( cstr[pos], column );
-                for ( size_t i = 0; i < replacement.length(); i++ ) {
-                    m_converted_pos_to_raw.push_back( pos);
-                    buffer += replacement[i];
-                    column++;
-                }
-                pos++;
-            }
-        } else {
-            cl = UTF8::getLenOfCharacter( cstr + pos );
+        storePosToRaw( pos, buffer.size(), column );
 
-            if ( cl <= 1 ) {
-                if ( UTF8::isValidCharacter( cstr + pos ) ) {
-                    m_converted_pos_to_raw.push_back( pos );
-                    buffer += cstr[pos++];
-                    column++;
-                } else {
-                    // invalid
-                    std::string replacement = getReplacementString( cstr[pos], column );
-                    for ( size_t i = 0; i < replacement.length(); i++ ) {
-                        m_converted_pos_to_raw.push_back( pos);
-                        buffer += replacement[i];
-                        column++;
-                    }
-                    pos++;
-                }
-            } else {
-                size_t base_pos = pos;
-                while ( cl > 0 && pos < len ) {
-                    m_converted_pos_to_raw.push_back( base_pos );
-                    buffer += cstr[pos++];
-                    cl--;
-                }
-                column++;
-            }
-        }
+        char buf[128];
+        size_t buf_rem = sizeof( buf );
+        char *out_buf = buf;
+        int res = convert_symbol( &cstr[pos], &column,
+                                  &out_buf, &buf_rem );
+        if ( res < 0 ) break;
+
+        buffer.append( buf, sizeof( buf ) - buf_rem );
+        
+        pos += res;
     }
-
-    m_converted_pos_to_raw.push_back( pos );
 }
 
 void TextStorageString::setConvertMode( enum text_storage_converion convert )
@@ -600,3 +558,131 @@ void TextStorageString::setConvertMode( enum text_storage_converion convert )
 
     initBuffer( m_current_line_limit );
 }
+
+size_t TextStorageString::getRawPosition( const size_t pos ) const
+{
+    std::tuple< size_t, size_t, uint8_t > current = { 0, 0, 0 };
+
+    for ( const auto &e : m_converted_pos_to_raw_snapshots ) {
+        if ( std::get<0>( e ) <= pos ) {
+            current = e;
+        } else {
+            break;
+        }
+    }
+
+    // for fast forward search use previous result if it is nearer than current
+    if ( std::get<0>( current ) < std::get<0>( m_last_raw_lookup ) &&
+         std::get<0>( m_last_raw_lookup ) <= pos ) {
+        current = m_last_raw_lookup;
+    }
+
+    if ( std::get<0>( current ) == pos ) {
+        return std::get<1>( current );
+    }
+
+    size_t raw_len = m_raw_buffer.length();
+    size_t raw_pos = std::get<1>( current );
+    const char *raw_str = m_raw_buffer.c_str();
+
+    uint8_t column = std::get<2>( current );
+
+    while ( raw_pos < raw_len ) {
+        char buf[128];
+        size_t buf_rem = sizeof( buf );
+        char *out_buf = buf;
+        uint8_t last_column = column;
+        int res = convert_symbol( &raw_str[raw_pos], &column,
+                                  &out_buf, &buf_rem );
+        if ( res < 0 ) break;
+
+        size_t last_pos = std::get<0>( current );
+
+        std::get<0>( current ) += sizeof( buf ) - buf_rem;
+        if ( std::get<0>( current ) > pos ) {
+            m_last_raw_lookup = std::make_tuple( last_pos, raw_pos, last_column );
+            return raw_pos;
+        }
+        
+        raw_pos += res;
+    }
+
+    m_last_raw_lookup = std::make_tuple( pos, raw_pos, column );
+
+    return raw_pos;
+}
+
+void TextStorageString::storePosToRaw( size_t raw_pos, size_t converted_pos, uint8_t column )
+{
+    if ( converted_pos >= ( m_converted_pos_to_raw_snapshots.size() + 1 ) * m_pos_to_raw_stepping ) {
+        m_converted_pos_to_raw_snapshots.push_back( std::make_tuple( converted_pos, raw_pos, column ) );
+    }
+}
+
+
+int TextStorageString::convert_symbol( const char *in, uint8_t *column,
+                                       char **out, size_t *out_rem ) const
+{
+    if ( *out_rem < 1 ) return -EINVAL;
+
+    if ( *in >= 0 && *in <= 127 ) {
+        if ( std::isprint( *in ) ) {
+            *((*out)++) = *in;
+            *column = ( *column + 1 ) % 8;
+            (*out_rem)--;
+            return 1;
+        } else if ( *in == '\n' ) {
+            *((*out)++) = *in;
+            *column = 0;
+            (*out_rem)--;
+            return 1;
+        } else {
+            std::string replacement = getReplacementString( *in, *column );
+            if ( replacement.length() > *out_rem ) {
+                return -ENOSPC;
+            }
+            for ( size_t i = 0; i < replacement.length(); i++ ) {
+                *((*out)++) = replacement[i];
+                *column = ( *column + 1 ) % 8;
+                (*out_rem)--;
+            }
+            return 1;
+        }
+    } else {
+        size_t cl = UTF8::getLenOfCharacter( in );
+
+        if ( cl <= 1 ) {
+            if ( UTF8::isValidCharacter( in ) ) {
+                *((*out)++) = *in;
+                *column = ( *column + 1 ) % 8;
+                (*out_rem)--;
+                return 1;
+            } else {
+                // invalid
+                std::string replacement = getReplacementString( *in, *column );
+                if ( replacement.length() > *out_rem ) {
+                    return -ENOSPC;
+                }
+                for ( size_t i = 0; i < replacement.length(); i++ ) {
+                    *((*out)++) = replacement[i];
+                    *column = ( *column + 1 ) % 8;
+                    (*out_rem)--;
+                }
+                return 1;
+            }
+        } else {
+            if ( cl > *out_rem ) {
+                return -ENOSPC;
+            }
+            size_t res = cl;
+            while ( cl > 0 ) {
+                *((*out)++) = *in++;
+                *column = ( *column + 1 ) % 8;
+                (*out_rem)--;
+                cl--;
+            }
+
+            return res;
+        }
+    }
+}
