@@ -1,6 +1,6 @@
 /* argclass.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2011 Ralf Hoffmann.
+ * Copyright (C) 2011-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -24,3 +24,20 @@
 ArgClass::~ArgClass()
 {
 }
+
+std::pair< IntArg, bool > StringArg::convertToInt() const
+{
+    const char *s1;
+    char *s2;
+
+    s1 = m_value.c_str();
+
+    auto l = strtol( s1, &s2, 10 );
+
+    if ( *s1 != '\0' && *s2 == '\0' &&
+         l >= INT32_MIN && l <= INT32_MAX ) {
+        return std::make_pair( (int)l, true );
+    }
+
+    return std::make_pair( 0, false );
+}
