@@ -1,6 +1,6 @@
 /* basic_actions.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2020 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -65,7 +65,7 @@ DNDAction *DNDAction::duplicate() const
   return dnda;
 }
 
-int DNDAction::run( WPUContext *wpu, ActionMessage *msg )
+int DNDAction::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     // do nothing in case of DND
     if(msg->mode!=msg->AM_MODE_DNDACTION) {
@@ -201,7 +201,7 @@ DoubleClickAction *DoubleClickAction::duplicate() const
   return ta;
 }
 
-int DoubleClickAction::run( WPUContext *wpu, ActionMessage *msg )
+int DoubleClickAction::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     normalmodeddaction( msg );
     return 0;
@@ -360,7 +360,7 @@ ShowAction *ShowAction::duplicate() const
   return ta;
 }
 
-int ShowAction::run( WPUContext *wpu, ActionMessage *msg )
+int ShowAction::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     normalmodeshow( msg );
     return 0;
@@ -519,7 +519,7 @@ RawShowAction *RawShowAction::duplicate() const
   return ta;
 }
 
-int RawShowAction::run( WPUContext *wpu, ActionMessage *msg )
+int RawShowAction::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     normalmoderawshow( msg );
     return 0;
@@ -681,7 +681,7 @@ UserAction *UserAction::duplicate() const
   return ta;
 }
 
-int UserAction::run( WPUContext *wpu, ActionMessage *msg )
+int UserAction::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     normalmodeuseraction( msg );
     return 0;
@@ -863,6 +863,15 @@ int UserAction::configure()
           if(msg->button.button==okb) endmode=0;
           else if(msg->button.button==cb) endmode=1;
           break;
+        case AG_KEYPRESSED:
+            if ( win->isParent( msg->key.window, false ) == true ) {
+                switch ( msg->key.key ) {
+                    case XK_Escape:
+                        endmode = 1;
+                        break;
+                }
+            }
+            break;
       }
       aguix->ReplyMessage(msg);
     }
@@ -946,7 +955,7 @@ const char *ChangeHiddenFlag::getName()
   return name;
 }
 
-int ChangeHiddenFlag::run( WPUContext *wpu, ActionMessage *msg )
+int ChangeHiddenFlag::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   ListerMode *lm1;
   if(msg->mode!=msg->AM_MODE_DNDACTION) {
@@ -1062,6 +1071,15 @@ int ChangeHiddenFlag::configure()
           if(msg->button.button==okb) endmode=0;
           else if(msg->button.button==cb) endmode=1;
           break;
+        case AG_KEYPRESSED:
+            if ( win->isParent( msg->key.window, false ) == true ) {
+                switch ( msg->key.key ) {
+                    case XK_Escape:
+                        endmode = 1;
+                        break;
+                }
+            }
+            break;
       }
       aguix->ReplyMessage(msg);
     }
