@@ -29,7 +29,7 @@
 #include <sys/time.h>
 #include "bookmarkdb_scanner.hh"
 #include "datei.h"
-#include <aguix/lowlevelfunc.h>
+#include "aguix/lowlevelfunc.h"
 
 BookmarkDB::BookmarkDB( const std::string &filename ) : m_filename( filename )
 {
