@@ -1,6 +1,6 @@
 /* changelabelop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2008-2014 Ralf Hoffmann.
+ * Copyright (C) 2008-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -70,7 +70,7 @@ const char *ChangeLabelOp::getName()
     return name;
 }
 
-int ChangeLabelOp::run( WPUContext *wpu, ActionMessage *msg )
+int ChangeLabelOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     Lister *l1 = NULL;
     ListerMode *lm1 = NULL;
