@@ -20,11 +20,11 @@
  */
 
 #include "condparser.h"
-#include <aguix/lowlevelfunc.h>
-#include <aguix/awindow.h>
-#include <aguix/acontainer.h>
-#include <aguix/fieldlistview.h>
-#include <aguix/button.h>
+#include "aguix/lowlevelfunc.h"
+#include "aguix/awindow.h"
+#include "aguix/acontainer.h"
+#include "aguix/fieldlistview.h"
+#include "aguix/button.h"
 #include "execlass.h"
 #include "fileentry.hh"
 #include "worker.h"
