@@ -1,6 +1,6 @@
 /* copycore.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2013-2023 Ralf Hoffmann.
+ * Copyright (C) 2013-2024 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -29,6 +29,7 @@
 #include "verzeichnis.hh"
 #include "aguix/lowlevelfunc.h"
 #include "aguix/util.h"
+#include "aguix/utf8.hh"
 #include <chrono>
 #include <ratio>
 #include "nwc_path.hh"
@@ -104,6 +105,12 @@ CopyCore::nm_newname_t CopyCore::getNewName4Dir( const FileEntry *oldfe, const c
 
             std::string return_str;
 
+            if ( ! UTF8::isValidCharacterString( newname ) ) {
+                char *newname2 = dupstring( UTF8::convertToValidCharacterString( newname ).c_str() );
+                _freesafe( newname );
+                newname = newname2;
+            }
+
             erg = m_copyorder->cowin->string_request( catalog.getLocale( 149 ),
                                                       textstr,
                                                       newname,
@@ -397,6 +404,12 @@ CopyCore::nm_newname_t CopyCore::getNewName4File( const FileEntry *oldfe, const
 
             std::string return_str;
 
+            if ( ! UTF8::isValidCharacterString( newname ) ) {
+                char *newname2 = dupstring( UTF8::convertToValidCharacterString( newname ).c_str() );
+                _freesafe( newname );
+                newname = newname2;
+            }
+
             erg = m_copyorder->cowin->string_request( catalog.getLocale( 149 ),
                                                     textstr,
                                                     newname,
@@ -982,6 +995,20 @@ CopyCore::copyfile( const FileEntry *fe, bool acceptrename, const char *destdir,
         if ( cowin != NULL ) cowin->conttimer();
     }
 
+    if ( ! fe->exists ) {
+        if ( cowin != NULL ) cowin->stoptimer();
+        myerrstr = strerror( errno );
+        text = AGUIXUtils::formatStringToString( catalog.getLocale( 1465 ), fe->fullname );
+        button = std::string( catalog.getLocale( 225 ) ) + "|" + catalog.getLocale( 8 );
+        erg = cowin->request( catalog.getLocale( 347 ), text.c_str(), button.c_str() );
+        if ( erg == 0 ) {
+            skip = true;
+        } else {
+            m_cancel = true;
+        }
+        if ( cowin != NULL ) cowin->conttimer();
+    }
+
     if ( m_cancel == false && skip == false ) {
         e1=getNewName4File(fe,destdir,&newname,acceptrename);
     
@@ -1944,6 +1971,7 @@ int CopyCore::executeCopyThread()
                     // success
                     Worker::pushCommandLog( m_copyorder->move ? catalog.getLocale( 1420 ) : catalog.getLocale( 1419 ),
                                             target_fullname,
+                                            cbe.entry().fullname,
                                             AGUIXUtils::formatStringToString( catalog.getLocale( 1421 ),
                                                                               cbe.entry().fullname,
                                                                               destdir.c_str() ) );
@@ -1979,6 +2007,7 @@ int CopyCore::executeCopyThread()
                     // success
                     Worker::pushCommandLog( m_copyorder->move ? catalog.getLocale( 1420 ) : catalog.getLocale( 1419 ),
                                             target_fullname,
+                                            cbe.entry().fullname,
                                             AGUIXUtils::formatStringToString( catalog.getLocale( 1421 ),
                                                                               cbe.entry().fullname,
                                                                               destdir.c_str() ) );
