@@ -1,6 +1,6 @@
 /* copyop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2021 Ralf Hoffmann.
+ * Copyright (C) 2001-2022 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -209,9 +209,41 @@ CopyOp::normalmodecopy( ActionMessage *am )
     tvdir_preserve_dir_structure = vdir_preserve_dir_structure;
     tadjust_relative_symlinks = adjust_relative_symlinks;
     tensure_file_permissions = ensure_file_permissions;
+
+    if ( toverwrite != COPYOP_OVERWRITE_NORMAL ) {
+        std::string textstr;
+
+        switch ( toverwrite ) {
+            case COPYOP_OVERWRITE_ALWAYS:
+                textstr = catalog.getLocale( 1439 );
+                break;
+            case COPYOP_OVERWRITE_NEVER:
+                textstr = catalog.getLocale( 1440 );
+                break;
+            case COPYOP_OVERWRITE_NORMAL:
+                // will not happen, but to make compiler happy
+                abort();
+                break;
+        }
+
+        std::string buttonstr = catalog.getLocale( 629 );
+        buttonstr += "|";
+        buttonstr += catalog.getLocale( 8 );
+
+        int erg = Worker::getRequester()->request( catalog.getLocale( 123 ),
+                                                   textstr.c_str(),
+                                                   buttonstr.c_str() );
+
+        if ( erg != 0 ) {
+            cont = false;
+        }
+    }
   }
   
-  if(cont==true) {
+  if ( ! cont ) {
+      return 0;
+  }
+
     std::shared_ptr< struct copyorder > co;
 
     co.reset( new copyorder );
@@ -395,8 +427,8 @@ CopyOp::normalmodecopy( ActionMessage *am )
             vdm->copy( co );
         }
     }
-  }
-  return 0;
+
+    return 0;
 }
 
 int
@@ -428,7 +460,7 @@ CopyOp::doconfigure(int mode)
   win->create();
   _freesafe(tstr);
 
-  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 12 ), true );
+  AContainer *ac1 = win->setContainer( new AContainer( win, 1, 13 ), true );
   ac1->setMinSpace( 5 );
   ac1->setMaxSpace( 5 );
 
@@ -454,7 +486,7 @@ CopyOp::doconfigure(int mode)
 
   AContainer *ac1_1 = ac1->add( new AContainer( win, 2, 1 ), 0, 7 );
   ac1_1->setMinSpace( 5 );
-  ac1_1->setMaxSpace( -1 );
+  ac1_1->setMaxSpace( 5 );
   ac1_1->setBorderWidth( 0 );
 
   ac1_1->addWidget( new Text( aguix, 0, 0, catalog.getLocale( 1088 ) ),
@@ -482,7 +514,7 @@ CopyOp::doconfigure(int mode)
 
   AContainer *ac1_ens_perm = ac1->add( new AContainer( win, 2, 1 ), 0, 8 );
   ac1_ens_perm->setMinSpace( 5 );
-  ac1_ens_perm->setMaxSpace( -1 );
+  ac1_ens_perm->setMaxSpace( 5 );
   ac1_ens_perm->setBorderWidth( 0 );
 
   ac1_ens_perm->addWidget( new Text( aguix, 0, 0, catalog.getLocale( 1097 ) ),
@@ -539,12 +571,40 @@ CopyOp::doconfigure(int mode)
   auto vdirpreserve_cb = ac1->addWidget( new ChooseButton( aguix, 0, 0, 20, 20, ( vdir_preserve_dir_structure == true ) ? 1 : 0,
                                                            catalog.getLocale( 1209 ), LABEL_RIGHT, 0 ), 0, 9, cincwnr );
 
-  if(mode==0) {
-    rfcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( requestParameters() == true ) ? 1 : 0,
-						      catalog.getLocale( 294 ), LABEL_RIGHT, 0 ), 0, 10, cincwnr );
+  AContainer *ac1_overwrite_mode = ac1->add( new AContainer( win, 2, 1 ), 0, 10 );
+  ac1_overwrite_mode->setMinSpace( 5 );
+  ac1_overwrite_mode->setMaxSpace( 5 );
+  ac1_overwrite_mode->setBorderWidth( 0 );
+
+  ac1_overwrite_mode->addWidget( new Text( aguix, 0, 0, catalog.getLocale( 303 ) ),
+                                 0, 0, cfix );
+  CycleButton *overwritemode_cyb = ac1_overwrite_mode->addWidget( new CycleButton( aguix, 0, 0, 100, 0 ),
+                                                                  1, 0, cincwnr );
+  overwritemode_cyb->addOption( catalog.getLocale( 304 ) );
+  overwritemode_cyb->addOption( catalog.getLocale( 305 ) );
+  overwritemode_cyb->addOption( catalog.getLocale( 306 ) );
+  overwritemode_cyb->resize( overwritemode_cyb->getMaxSize(),
+                             overwritemode_cyb->getHeight() );
+  ac1_overwrite_mode->readLimits();
+
+  switch ( overwrite ) {
+      case COPYOP_OVERWRITE_ALWAYS:
+          overwritemode_cyb->setOption( 1 );
+          break;
+      case COPYOP_OVERWRITE_NEVER:
+          overwritemode_cyb->setOption( 2 );
+          break;
+      default:
+          overwritemode_cyb->setOption( 0 );
+          break;
   }
 
-  AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, 11 );
+  if(mode==0) {
+    rfcb = (ChooseButton*)ac1->add( new ChooseButton( aguix, 0, 0, 20, 20, ( requestParameters() == true ) ? 1 : 0,
+						      catalog.getLocale( 294 ), LABEL_RIGHT, 0 ), 0, 11, cincwnr );
+  }
+
+  AContainer *ac1_2 = ac1->add( new AContainer( win, 2, 1 ), 0, 12 );
   ac1_2->setMinSpace( 5 );
   ac1_2->setMaxSpace( -1 );
   ac1_2->setBorderWidth( 0 );
@@ -605,6 +665,9 @@ CopyOp::doconfigure(int mode)
               case XK_9:
                   vdirpreserve_cb->setState( ( vdirpreserve_cb->getState() == true ) ? false : true );
                   break;
+              case XK_0:
+                  overwritemode_cyb->setOption( ( overwritemode_cyb->getSelectedOption() + 1 ) % overwritemode_cyb->getNrOfOptions() );
+                  break;
               case XK_Return:
                 if ( cb->getHasFocus() == false ) {
                   endmode = 0;
@@ -661,17 +724,17 @@ CopyOp::doconfigure(int mode)
       }
 
       toverwrite=COPYOP_OVERWRITE_NORMAL;
-/*      switch(ocyb->getSelectedOption()) {
-        case 1:
-          toverwrite=COPYOP_OVERWRITE_ALWAYS;
-          break;
-        case 2:
-          toverwrite=COPYOP_OVERWRITE_NEVER;
-          break;
-        default:
-          toverwrite=COPYOP_OVERWRITE_NORMAL;
-          break;
-      }*/
+      switch ( overwritemode_cyb->getSelectedOption() ) {
+          case 1:
+              toverwrite = COPYOP_OVERWRITE_ALWAYS;
+              break;
+          case 2:
+              toverwrite = COPYOP_OVERWRITE_NEVER;
+              break;
+          default:
+              toverwrite = COPYOP_OVERWRITE_NORMAL;
+              break;
+      }
     } else {
       // store in normal variables
       follow_symlinks = fscb->getState();
@@ -711,17 +774,17 @@ CopyOp::doconfigure(int mode)
       }
 
       overwrite=COPYOP_OVERWRITE_NORMAL;
-/*      switch(ocyb->getSelectedOption()) {
-        case 1:
-          overwrite=COPYOP_OVERWRITE_ALWAYS;
-          break;
-        case 2:
-          overwrite=COPYOP_OVERWRITE_NEVER;
-          break;
-        default:
-          overwrite=COPYOP_OVERWRITE_NORMAL;
-          break;
-      }*/
+      switch ( overwritemode_cyb->getSelectedOption() ) {
+          case 1:
+              overwrite = COPYOP_OVERWRITE_ALWAYS;
+              break;
+          case 2:
+              overwrite = COPYOP_OVERWRITE_NEVER;
+              break;
+          default:
+              overwrite = COPYOP_OVERWRITE_NORMAL;
+              break;
+      }
     }
   }
   
