@@ -1,6 +1,6 @@
 /* copyopwin.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2014 Ralf Hoffmann.
+ * Copyright (C) 2001-2024 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -337,8 +337,7 @@ CopyOpWin::redraw( bool skip_message_handling )
   double drtime;
   struct timeval curtime;
   double difftime2, lastdiff2;
-  char tstr[1024],
-       *tstr2;
+  char tstr[1024];
   double rate;
   int tw;
   bool doresize = false;
@@ -348,21 +347,13 @@ CopyOpWin::redraw( bool skip_message_handling )
               copied_bytes_str,
               bytes_str;
   
-  char *tstr3;
-
     std::unique_lock< std::recursive_mutex > lck( m_status_update_lock );
 
   timerclear( &curtime );
   if ( update_sourcetext == true ) {
       if ( ismessage[0] == false ) {
-          tstr3 = Datei::shrinkFilename( source.c_str(), filenamespace[0], *lencalc );
-          if ( tstr3 == NULL)
-              tstr3 = dupstring( source.c_str() );
-          tstr2 = (char*)_allocsafe( strlen( catalog.getLocale( ( move == true ) ? 594 : 116 ) ) + strlen( tstr3 ) + 1 );
-          sprintf( tstr2, catalog.getLocale( ( move == true ) ? 594 : 116 ), tstr3 );
-          _freesafe( tstr3 );
-          sourcetext->setText(tstr2);
-          _freesafe(tstr2);
+          auto shrinked_source = m_fns.shrink( source, filenamespace[0], *lencalc );
+          sourcetext->setText( AGUIXUtils::formatStringToString( catalog.getLocale( ( move == true ) ? 594 : 116 ), shrinked_source.c_str() ).c_str() );
           /* no need for explicit redraw */
       } else {
           sourcetext->setText( source.c_str() );
@@ -371,14 +362,8 @@ CopyOpWin::redraw( bool skip_message_handling )
   }
   if ( update_desttext == true ) {
       if ( ismessage[1] == false ) {
-          tstr3 = Datei::shrinkFilename( dest.c_str(), filenamespace[1], *lencalc );
-          if ( tstr3 == NULL)
-              tstr3 = dupstring( dest.c_str() );
-          tstr2 = (char*)_allocsafe( strlen( catalog.getLocale( 117 ) ) + strlen( tstr3 ) + 1 );
-          sprintf( tstr2, catalog.getLocale( 117 ), tstr3 );
-          _freesafe(tstr3);
-          desttext->setText(tstr2);
-          _freesafe(tstr2);
+          auto shrinked_dest = m_fns.shrink( dest, filenamespace[1], *lencalc );
+          desttext->setText( AGUIXUtils::formatStringToString( catalog.getLocale( 117 ), shrinked_dest.c_str() ).c_str() );
           // no need for explicit redraw
       } else {
           desttext->setText( dest.c_str() );
@@ -525,10 +510,10 @@ CopyOpWin::redraw( bool skip_message_handling )
     newglob_percent = (int)( rate * 100.0 );
     if ( newglob_percent != glob_percent ) {
         glob_percent = newglob_percent;
-        tstr3 = (char*)_allocsafe( strlen( ( move == true ) ? catalog.getLocale( 299 ) : catalog.getLocale( 1249 ) ) +
-                                   strlen( " ( %)" ) + 
-                                   A_BYTESFORNUMBER( glob_percent ) +
-                                   1 );
+        char *tstr3 = (char*)_allocsafe( strlen( ( move == true ) ? catalog.getLocale( 299 ) : catalog.getLocale( 1249 ) ) +
+                                         strlen( " ( %)" ) + 
+                                         A_BYTESFORNUMBER( glob_percent ) +
+                                         1 );
         if ( ( glob_percent > 0 ) && ( glob_percent <= 100 ) ) {
             sprintf( tstr3, "%s (%d %%)",
                      ( move == true ) ? catalog.getLocale( 299 ) : catalog.getLocale( 1249 ),
