@@ -20,10 +20,10 @@
  */
 
 #include "dircomparewin.hh"
-#include <aguix/awindow.h>
-#include <aguix/text.h>
-#include <aguix/button.h>
-#include <aguix/acontainerbb.h>
+#include "aguix/awindow.h"
+#include "aguix/text.h"
+#include "aguix/button.h"
+#include "aguix/acontainerbb.h"
 #include "worker_locale.h"
 #include "filenameshrinker.hh"
 
