@@ -172,13 +172,16 @@ bool DirFilterSettings::check( const std::string &dirname,
                         }
                         if ( label_it != label_colors.end() ) {
                             WConfig::ColorDef::label_colors_t col = label_it->second;
-                            es.setOverrideColor( FileEntryCustomColor::NORMAL,
-                                                 col.normal_fg,
-                                                 col.normal_bg );
-                            es.setOverrideColor( FileEntryCustomColor::ACTIVE,
-                                                 col.active_fg,
-                                                 col.active_bg );
-                            clear_override = false;
+
+                            if ( ! col.only_exact_path || es.getBookmarkMatch() == FileEntry::BOOKMARK ) {
+                                es.setOverrideColor( FileEntryCustomColor::NORMAL,
+                                                     col.normal_fg,
+                                                     col.normal_bg );
+                                es.setOverrideColor( FileEntryCustomColor::ACTIVE,
+                                                     col.active_fg,
+                                                     col.active_bg );
+                                clear_override = false;
+                            }
                         }
                     }
                 } catch ( int ) {
