@@ -143,7 +143,7 @@ GoFTPOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
             if ( lm1 != NULL ) {
                 std::list< RefCount< ArgClass > > args;
 
-                args.push_back( new StringArg( str1 ) );
+                args.push_back( RefCount< ArgClass >( new StringArg( str1 ) ) );
                 lm1->runCommand( "enter_dir", args );
             }
 	}
