@@ -1,6 +1,6 @@
 /* intviewop.h
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2005-2017 Ralf Hoffmann.
+ * Copyright (C) 2005-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -35,16 +35,13 @@ public:
   InternalViewOp( const InternalViewOp &other );
   InternalViewOp &operator=( const InternalViewOp &other );
 
-  InternalViewOp *duplicate() const;
-  virtual bool isName(const char *);
-  virtual const char *getName();
-#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
-  using FunctionProto::run;
-#endif
-  int run( WPUContext*, ActionMessage* );
-  bool save(Datei*);
-  virtual const char *getDescription();
-  int configure();
+  InternalViewOp *duplicate() const override;
+  bool isName(const char *) override;
+  const char *getName() override;
+  int run( std::shared_ptr< WPUContext > wpu, ActionMessage* ) override;
+  bool save(Datei*) override;
+  const char *getDescription() override;
+  int configure() override;
   
   int doconfigure( int mode );
   
