@@ -25,12 +25,12 @@
 #include "wconfig.h"
 #include "configheader.h"
 #include "configparser.hh"
-#include <aguix/acontainer.h>
-#include <aguix/button.h>
-#include <aguix/choosebutton.h>
-#include <aguix/fieldlistview.h>
-#include <aguix/awindow.h>
-#include <aguix/icons.hh>
+#include "aguix/acontainer.h"
+#include "aguix/button.h"
+#include "aguix/choosebutton.h"
+#include "aguix/fieldlistview.h"
+#include "aguix/awindow.h"
+#include "aguix/icons.hh"
 #include "listermode.h"
 #include "datei.h"
 #include "nmfilter.hh"
