@@ -45,7 +45,7 @@ void MagicDB::init()
 #ifdef HAVE_LIBMAGIC
     m_magic_cookie = magic_open( MAGIC_NONE );
     if ( m_magic_cookie == NULL ) return;
-    magic_load( m_magic_cookie, NULL );
+    magic_load( m_magic_cookie, magic_getpath( NULL, 0 ) );
 #endif
     m_magic_init = true;
 }
