@@ -1,6 +1,6 @@
 /* menutreeui.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2013-2014,2017 Ralf Hoffmann.
+ * Copyright (C) 2013-2023 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -69,7 +69,30 @@ void MenuTreeUI::setMenuTree( MenuTree *menu )
     m_menu = menu;
 }
 
-void MenuTreeUI::show( bool show_recently_used )
+static MenuTreeNode *findNode( MenuTreeNode *start_node,
+                               const MenuTreeNode *node_to_search )
+{
+    if ( ! start_node ||
+         ! node_to_search ) {
+        return NULL;
+    }
+
+    for ( const auto &node : *start_node ) {
+        if ( node == node_to_search ) {
+            return node;
+        }
+        if ( node->hasChildren() ) {
+            auto res = findNode( node, node_to_search );
+            if ( res ) {
+                return res;
+            }
+        }
+    }
+
+    return NULL;
+}
+
+void MenuTreeUI::show( bool show_recently_used, MenuTreeNode *initial_node )
 {
     const AWindow *twin = NULL;
 
@@ -90,7 +113,7 @@ void MenuTreeUI::show( bool show_recently_used )
     maincont->setMaxSpace( 5 );
 
     RefCount<AFontWidth> lencalc( new AFontWidth( m_aguix, NULL ) );
-    TextStorageString help_ts( catalog.getLocale( 1007 ), lencalc );
+    auto help_ts = std::make_shared< TextStorageString >( catalog.getLocale( 1007 ), lencalc );
     TextView *help_tv = (TextView*)maincont->add( new TextView( m_aguix,
                                                                 0, 0, 50, 80, "", help_ts ),
                                                   0, 0, AContainer::CO_INCW );
@@ -205,7 +228,17 @@ void MenuTreeUI::show( bool show_recently_used )
     AGMessage *agmsg;
 
     MenuTreeNode *root = m_menu->getRootNode();
-    m_menu->setCurrentNode( root );
+
+    if ( initial_node ) {
+        auto node = findNode( root, initial_node );
+        if ( node ) {
+            m_menu->setCurrentNode( node );
+        } else {
+            m_menu->setCurrentNode( root );
+        }
+    } else {
+        m_menu->setCurrentNode( root );
+    }
 
     m_show_recently_used = show_recently_used;
 
@@ -218,6 +251,7 @@ void MenuTreeUI::show( bool show_recently_used )
     m_menu->setFilter( "" );
 
     showMenu();
+    updateBreadcrumb();
 
     std::string current_filter;
 
