@@ -135,22 +135,22 @@ ModifyTabsOp::nm_tab_action( ActionMessage *am )
                     lm1->runCommand( "close_current_tab" );
                     break;
                 case NEXT_TAB:
-                    args.push_back( new IntArg( 1 ) );
+                    args.push_back( RefCount< ArgClass >( new IntArg( 1 ) ) );
                     lm1->runCommand( "switch_to_tab_direction", args );
                     break;
                 case PREV_TAB:
-                    args.push_back( new IntArg( -1 ) );
+                    args.push_back( RefCount< ArgClass >( new IntArg( -1 ) ) );
                     lm1->runCommand( "switch_to_tab_direction", args );
                     break;
                 case TOGGLE_LOCK_STATE:
                     lm1->runCommand( "toggle_lock_current_tab" );
                     break;
                 case MOVE_TAB_LEFT:
-                    args.push_back( new IntArg( -1 ) );
+                    args.push_back( RefCount< ArgClass >( new IntArg( -1 ) ) );
                     lm1->runCommand( "move_tab_direction", args );
                     break;
                 case MOVE_TAB_RIGHT:
-                    args.push_back( new IntArg( 1 ) );
+                    args.push_back( RefCount< ArgClass >( new IntArg( 1 ) ) );
                     lm1->runCommand( "move_tab_direction", args );
                     break;
                 default:
