@@ -26,6 +26,7 @@
 #include "nwc_path.hh"
 #include "nwc_os.hh"
 #include "aguix/lowlevelfunc.h"
+#include "error_count.hh"
 
 namespace NWC
 {
@@ -59,6 +60,8 @@ namespace NWC
         _statbuf.dev = stbuf.st_dev;
 
         _exists = true;
+      } else {
+          ErrorCount::account_errno( errno );
       }
 
       if ( S_ISLNK( _statbuf.mode ) ) {
@@ -79,6 +82,7 @@ namespace NWC
           _dest_statbuf.dev = stbuf.st_dev;
         } else {
           /* corrupt */
+          ErrorCount::account_errno( errno );
           _is_corrupt = true;
           _dest_statbuf.mode = 0;  // to prevent corrupted links treated as dirs
           if ( errno == ENOENT ) {
@@ -456,6 +460,8 @@ namespace NWC
                 _statbuf.mode |= stbuf.st_mode & (S_ISUID|S_ISGID|S_ISVTX|S_IRWXU|S_IRWXG|S_IRWXO);
 
                 _exists = true;
+            } else {
+                ErrorCount::account_errno( errno );
             }
 
             if ( S_ISLNK( _statbuf.mode ) ) {
@@ -478,6 +484,7 @@ namespace NWC
                     _dest_statbuf.mode |= stbuf.st_mode & (S_ISUID|S_ISGID|S_ISVTX|S_IRWXU|S_IRWXG|S_IRWXO);
                 } else {
                     /* corrupt */
+                    ErrorCount::account_errno( errno );
                     _is_corrupt = true;
                     _dest_statbuf.mode = 0;  // to prevent corrupted links treated as dirs
                     if ( errno == ENOENT ) {
