@@ -1,6 +1,6 @@
 /* opencontextmenuop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2007-2015 Ralf Hoffmann.
+ * Copyright (C) 2007-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -61,7 +61,7 @@ const char *OpenContextMenuOp::getName()
     return name;
 }
 
-int OpenContextMenuOp::run( WPUContext *wpu, ActionMessage *msg )
+int OpenContextMenuOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
     Lister *l1;
     ListerMode *lm1;
@@ -150,6 +150,15 @@ int OpenContextMenuOp::configure()
                     if ( msg->button.button == okb ) endmode = 0;
                     else if ( msg->button.button == cancelb ) endmode = 1;
                     break;
+                case AG_KEYPRESSED:
+                    if ( win->isParent( msg->key.window, false ) == true ) {
+                        switch ( msg->key.key ) {
+                            case XK_Escape:
+                                endmode = 1;
+                                break;
+                        }
+                    }
+                    break;
             }
             aguix->ReplyMessage( msg );
         }
