@@ -23,8 +23,8 @@
 #define PARTSPACE_H
 
 #include "wdefines.h"
-#include <aguix/mutex.h>
-#include <aguix/condvar.h>
+#include "aguix/mutex.h"
+#include "aguix/condvar.h"
 #include <list>
 #include <map>
 #include <string>
