@@ -1,6 +1,6 @@
 /* searchopbg.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2014 Ralf Hoffmann.
+ * Copyright (C) 2006-2025 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -25,6 +25,7 @@
 #include "worker.h"
 #include "execlass.h"
 #include "nwc_virtualdir.hh"
+#include "nwc_path.hh"
 #include "fileviewer.hh"
 #include "flagreplacer.hh"
 #include "aguix/acontainerbb.h"
@@ -40,6 +41,9 @@
 #include "resultstore.hh"
 #include "virtualdirmode.hh"
 #include "argclass.hh"
+#include "worker_requests.hh"
+#include "bookmarkdbproxy.hh"
+#include "bookmarkdbentry.hh"
 
 static const int searchopbg_timeout = 10;
 
@@ -260,6 +264,8 @@ void SearchOpBG::prepareNewSearch()
     _resultlv->setSize( 0 );
     _resultlv->redraw();
 
+    m_skip_prefixes.clear();
+
     prepareStartCont();
 }
 
@@ -280,9 +286,14 @@ void SearchOpBG::buildStoreLV()
 
             sprintf( buf, "%d", (int)( (*it)->getRes()->size() ) );
             _storelv->setText( row, 1, buf );
-            _storelv->setText( row, 2, sets.getMatchName() );
-            _storelv->setText( row, 3, sets.getMatchContent() );
-            _storelv->setText( row, 4, sets.getBaseDir() );
+            _storelv->setText( row, 2, AGUIXUtils::formatStringToString( "%" PRIu64,
+                                                                         (uint64_t)(*it)->get_excluded() ) );
+            _storelv->setText( row, 3, AGUIXUtils::formatStringToString( "%" PRIu64,
+                                                                         (uint64_t)(*it)->get_removed() ) );
+            _storelv->setText( row, 4, sets.getMatchName() );
+            _storelv->setText( row, 5, sets.getMatchContent() );
+            _storelv->setText( row, 6, sets.getMatchExpression() );
+            _storelv->setText( row, 7, sets.getBaseDir() );
 
             if ( (*it)->is_exclusive() && *it != _rst ) {
                 _storelv->setText( row, 0, "L" );
@@ -410,6 +421,7 @@ void SearchOpBG::startSearch()
     SearchSettings sets = getCurrentSearchSettings();;
 
     std::unique_ptr<NWC::VirtualDir> use_dir( _rst->getClonedDir() );
+    std::string skip_initial_dir;
   
     if ( use_dir->size() < 1 ) {
         std::string d = sets.getBaseDir();
@@ -418,8 +430,10 @@ void SearchOpBG::startSearch()
                 use_dir->add( *m_initial_start_vdir );
             } else {
                 use_dir->add( NWC::VirtualDir( m_initial_start_dir ) );
+                skip_initial_dir = m_initial_start_dir;
             }
         } else {
+            skip_initial_dir = d;
             use_dir->add( NWC::VirtualDir( d ) );
         }
     } else {
@@ -434,7 +448,18 @@ void SearchOpBG::startSearch()
     DW_SetFollowSymlinks dw1( sets.getFollowSymlinks(), sets.getSearchSameFS() );
     use_dir->walk( dw1 );
 
-    _sth = new SearchThread( std::unique_ptr<NWC::Dir>( std::move( use_dir ) ) );
+    _sth = new SearchThread( std::unique_ptr<NWC::Dir>( std::move( use_dir ) ),
+                             skip_initial_dir );
+
+    std::list< std::string > exclude_paths;
+    if ( ! sets.getExcludeLabel().empty() ) {
+        for ( const auto &e : Worker::getBookmarkDBInstance().getEntries( sets.getExcludeLabel() ) ) {
+            exclude_paths.push_back( e.getName() );
+        }
+    }
+
+    _sth->setExcludePaths( exclude_paths );
+
     _sth->setSearchSettings( sets );
     _sth->start();
 
@@ -458,6 +483,18 @@ void SearchOpBG::stopSearch()
     }
 }
 
+static bool is_skipped( const std::list< std::string > &skip_prefixes,
+                        const std::string &fullname )
+{
+    for ( const auto &e : skip_prefixes ) {
+        if ( NWC::Path::is_prefix_dir( e, fullname ) ) {
+            return true;
+        }
+    }
+
+    return false;
+}
+
 void SearchOpBG::gatherResults( int get_number_of_results, int time_limit_ms )
 {
     // get results from thread and put them into rst
@@ -475,6 +512,15 @@ void SearchOpBG::gatherResults( int get_number_of_results, int time_limit_ms )
 
         while ( _sth->resultsAvailable() ) {
             SearchThread::SearchResult erg = _sth->getResultLocked();
+            size_t excluded_entries = _sth->getExcludedEntries();
+
+            _rst->set_excluded( excluded_entries );
+
+            if ( is_skipped( m_skip_prefixes, erg.getFullname() ) ) {
+                _rst->set_removed( _rst->get_removed() + 1 );
+                continue;
+            }
+
             _rst->getRes()->push_back( erg );
             if ( consumeResult( erg, *_resultlv ) == true ) {
                 _rst->getDir()->add( NWC::FSEntry( erg.getFullname() ) );
@@ -559,6 +605,7 @@ SearchSettings SearchOpBG::getCurrentSearchSettings()
 {
     SearchSettings sets( _startdir_sg->getText(),
                          _matchname_sg->getText(),
+                         m_matchname_separator_sg->getText(),
                          _matchname_regexp_cb->getState(),
                          _matchname_fullpath_cb->getState(),
                          _matchname_case_cb->getState(),
@@ -567,6 +614,8 @@ SearchSettings SearchOpBG::getCurrentSearchSettings()
                          _follow_symlinks_cb->getState(),
                          _search_vfs_cb->getState(),
                          _search_same_fs_cb->getState(),
+                         m_exclude_label_sg->getText(),
+                         m_match_expression_sg->getText(),
                          m_younger_cb->getState(),
                          m_younger_sg->getText(),
                          m_older_cb->getState(),
@@ -603,6 +652,7 @@ void SearchOpBG::restoreSearchSettings()
         _startdir_sg->setText( sets.getBaseDir().c_str() );
     }
     _matchname_sg->setText( sets.getMatchName().c_str() );
+    m_matchname_separator_sg->setText( sets.getMatchNameSeparator().c_str() );
     _matchname_regexp_cb->setState( sets.getMatchNameRE() );
     _matchname_fullpath_cb->setState( sets.getMatchNameFullPath() );
     _matchname_case_cb->setState( sets.getMatchNameCase() );
@@ -611,6 +661,10 @@ void SearchOpBG::restoreSearchSettings()
     _follow_symlinks_cb->setState( sets.getFollowSymlinks() );
     _search_vfs_cb->setState( sets.getSearchVFS() );
     _search_same_fs_cb->setState( sets.getSearchSameFS() );
+    m_match_expression_sg->setText( sets.getMatchExpression().c_str() );
+
+    m_exclude_label_sg->setText( sets.getExcludeLabel().c_str() );
+
     m_younger_cb->setState( sets.getCheckYounger() );
     m_younger_sg->setText( sets.getYoungerThan().c_str() );
     m_older_cb->setState( sets.getCheckOlder() );
@@ -625,6 +679,8 @@ void SearchOpBG::restoreSearchSettings()
     m_max_vfs_depth_sg->setText( s1.c_str() );
 
     m_also_dirs_cb->setState( sets.getMatchAlsoDirectories() );
+
+    updateExpressionHelp();
 }
 
 void SearchOpBG::setCurrentResultStore( int pos, const SearchSettings *sets )
@@ -653,9 +709,41 @@ void SearchOpBG::removeSelectedResults()
   
     _rst->removeEntries( *( selected_entries.get() ) );
     buildLV();
+    buildStoreLV();
     _resultlv->centerActive();
 }
 
+void SearchOpBG::removeByBaseDir()
+{
+    int row = _resultlv->getActiveRow();
+    if ( ! _resultlv->isValidRow( row ) ) return;
+
+    std::string path = _resultlv->getText( row, 2 );
+
+    WorkerRequests req( Worker::getAGUIX(), m_win );
+
+    auto res = req.request_path_component( catalog.getLocale( 1556 ),
+                                           catalog.getLocale( 1557 ),
+                                           path,
+                                           { catalog.getLocale( 11 ),
+                                             catalog.getLocale( 8 )
+                                           } );
+
+    if ( res.decision != 0 ) {
+        return;
+    }
+
+    _rst->removeEntriesIf( [&prefix = res.path]( auto &fullname, auto line_nr ) {
+        return NWC::Path::is_prefix_dir( prefix, fullname );
+    } );
+
+    buildLV();
+    buildStoreLV();
+    _resultlv->centerActive();
+
+    m_skip_prefixes.push_back( res.path );
+}
+
 std::unique_ptr< std::list< std::pair< std::string, int > > > SearchOpBG::getSelectedResults()
 {
     std::list<SearchThread::SearchResult> *res = _rst->getRes();
@@ -895,13 +983,13 @@ void SearchOpBG::createWindow()
     acsw1_1->setMaxSpace( 5 );
     acsw1_1->setBorderWidth( 5 );
 
-    _start_ac = acsw1_1->add( new AContainer( subwin1, 2, 3 ), 0, 0 );
+    _start_ac = acsw1_1->add( new AContainer( subwin1, 2, 5 ), 0, 0 );
     _start_ac->setMinSpace( 5 );
     _start_ac->setMaxSpace( 5 );
     _start_ac->setBorderWidth( 0 );
 
-    _start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 722 ) ), 0, 0, AContainer::CO_FIX );
-    _startdir_text = (Text*)_start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 723 ) ), 1, 0, AContainer::CO_INCW );
+    _start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 722 ) ), 0, 0, AContainer::CO_FIXNR + AContainer::ACONT_CENTERV );
+    _startdir_text = (Text*)_start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 723 ), Text::CENTER_VERTICALLY ), 1, 0, AContainer::CO_INCW );
     _startdir_text->hide();
     _startdir_sg = (StringGadget*)_start_ac->add( new StringGadget( aguix, 0, 0, 100,
                                                                     m_initial_start_vdir.get() != NULL ?
@@ -910,19 +998,24 @@ void SearchOpBG::createWindow()
                                                                     0 ), 1, 0, AContainer::CO_INCW );
     _startdir_text->resize( _startdir_text->getWidth(), _startdir_sg->getHeight() );
 
-    AContainer *ac1_2 = _start_ac->add( new AContainer( subwin1, 4, 1 ), 1, 1 );
-    ac1_2->setMinSpace( 0 );
-    ac1_2->setMaxSpace( 0 );
+    AContainer *ac1_2 = _start_ac->add( new AContainer( subwin1, 6, 1 ), 1, 1 );
+    ac1_2->setMinSpace( 5 );
+    ac1_2->setMaxSpace( 5 );
     ac1_2->setBorderWidth( 0 );
 
-    _start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 724 ) ), 0, 1, AContainer::CO_FIX );
+    _start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 724 ) ), 0, 1, AContainer::CO_FIXNR + AContainer::ACONT_CENTERV );
+
     _matchname_sg = (StringGadget*)ac1_2->add( new StringGadget( aguix, 0, 0, 100, "*", 0 ), 0, 0, AContainer::CO_INCW );
+    ac1_2->addWidget( new Text( aguix, 0, 0, catalog.getLocale( 1559 ) ), 1, 0, AContainer::CO_FIXNR + AContainer::ACONT_CENTER );
+    m_matchname_separator_sg = ac1_2->addWidget( new StringGadget( aguix, 0, 0, aguix->getTextWidth( "MMM" ) + 8,
+                                                                   ",",
+                                                                   0 ), 2, 0, AContainer::CO_FIX );
     _matchname_regexp_cb = (ChooseButton*)ac1_2->add( new ChooseButton( aguix, 0, 0, false,
-                                                                        catalog.getLocale( 502 ), LABEL_RIGHT, 0 ), 1, 0, AContainer::CO_FIXNR );
+                                                                        catalog.getLocale( 502 ), LABEL_RIGHT, 0 ), 3, 0, AContainer::CO_FIXNR );
     _matchname_fullpath_cb = (ChooseButton*)ac1_2->add( new ChooseButton( aguix, 0, 0, false,
-                                                                          catalog.getLocale( 570 ), LABEL_RIGHT, 0 ), 2, 0, AContainer::CO_FIXNR );
+                                                                          catalog.getLocale( 570 ), LABEL_RIGHT, 0 ), 4, 0, AContainer::CO_FIXNR );
     _matchname_case_cb = (ChooseButton*)ac1_2->add( new ChooseButton( aguix, 0, 0, false,
-                                                                      catalog.getLocale( 725 ), LABEL_RIGHT, 0 ), 3, 0, AContainer::CO_FIXNR );
+                                                                      catalog.getLocale( 725 ), LABEL_RIGHT, 0 ), 5, 0, AContainer::CO_FIXNR );
     _matchname_regexp_cb->resize( _matchname_sg->getHeight(), _matchname_sg->getHeight() );
     _matchname_fullpath_cb->resize( _matchname_sg->getHeight(), _matchname_sg->getHeight() );
     _matchname_case_cb->resize( _matchname_sg->getHeight(), _matchname_sg->getHeight() );
@@ -933,29 +1026,57 @@ void SearchOpBG::createWindow()
     ac1_3->setMaxSpace( 0 );
     ac1_3->setBorderWidth( 0 );
 
-    _start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 726 ) ), 0, 2, AContainer::CO_FIX );
+    _start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 726 ) ), 0, 2, AContainer::CO_FIXNR + AContainer::ACONT_CENTERV );
     _matchcontent_sg = (StringGadget*)ac1_3->add( new StringGadget( aguix, 0, 0, 100, "", 0 ), 0, 0, AContainer::CO_INCW );
     _matchcontent_case_cb = (ChooseButton*)ac1_3->add( new ChooseButton( aguix, 0, 0, false,
                                                                          catalog.getLocale( 725 ), LABEL_RIGHT, 0 ), 1, 0, AContainer::CO_FIXNR );
     _matchcontent_case_cb->resize( _matchcontent_sg->getHeight(), _matchcontent_sg->getHeight() );
     ac1_3->readLimits();
 
-    AContainer *ac1_globsets = acsw1_1->add( new AContainer( subwin1, 5, 1 ), 0, 1 );
+    m_expression_ac = _start_ac->add( new AContainer( subwin1, 2, 1 ), 1, 3 );
+    m_expression_ac->setMinSpace( 5 );
+    m_expression_ac->setMaxSpace( 5 );
+    m_expression_ac->setBorderWidth( 0 );
+
+    _start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 1570 ) ), 0, 3, AContainer::CO_FIXNR + AContainer::ACONT_CENTERV );
+    m_match_expression_sg = m_expression_ac->addWidget( new StringGadget( aguix, 0, 0, 100, "", 0 ), 0, 0, AContainer::ACONT_MINH + AContainer::ACONT_MAXH );
+    m_expression_valid_t = m_expression_ac->addWidget( new Text( aguix, 0, 0, "" ), 1, 0, AContainer::CO_FIX );
+
+    _start_ac->add( new Text( aguix, 0, 0, catalog.getLocale( 1571 ) ), 0, 4, AContainer::CO_FIXNR + AContainer::ACONT_CENTERV );
+    m_expression_help_ac = _start_ac->add( new AContainer( subwin1, 16, 1 ), 1, 4 );
+    m_expression_help_ac->setMinSpace( 5 );
+    m_expression_help_ac->setMaxSpace( 5 );
+    m_expression_help_ac->setBorderWidth( 0 );
+
+    for ( size_t p = 0; p < 15; p++ ) {
+        Button *b = m_expression_help_ac->addWidget( new Button( Worker::getAGUIX(),
+                                                                 0,
+                                                                 0,
+                                                                 "",
+                                                                 0 ), p, 0, AContainer::CO_FIX );
+        m_expression_help_buttons.push_back( b );
+        m_expression_help_token.push_back( "" );
+    }
+
+    AContainer *ac1_globsets = acsw1_1->add( new AContainer( subwin1, 7, 1 ), 0, 1 );
     ac1_globsets->setMinSpace( 5 );
     ac1_globsets->setMaxSpace( 5 );
     ac1_globsets->setBorderWidth( 0 );
 
     _follow_symlinks_cb = (ChooseButton*)ac1_globsets->add( new ChooseButton( aguix, 0, 0, false,
-                                                                              catalog.getLocale( 298 ), LABEL_RIGHT, 0 ), 0, 0, AContainer::CO_FIXNR );
+                                                                              catalog.getLocale( 298 ), LABEL_RIGHT, 0 ), 0, 0, AContainer::CO_FIXNR + AContainer::ACONT_CENTERV );
     _search_vfs_cb = (ChooseButton*)ac1_globsets->add( new ChooseButton( aguix, 0, 0, false,
-                                                                         catalog.getLocale( 727 ), LABEL_RIGHT, 0 ), 1, 0, AContainer::CO_FIXNR );
+                                                                         catalog.getLocale( 727 ), LABEL_RIGHT, 0 ), 1, 0, AContainer::CO_FIXNR + AContainer::ACONT_CENTERV );
     _search_same_fs_cb = (ChooseButton*)ac1_globsets->add( new ChooseButton( aguix, 0, 0, false,
-                                                                             catalog.getLocale( 792 ), LABEL_RIGHT, 0 ), 2, 0, AContainer::CO_FIXNR );
+                                                                             catalog.getLocale( 792 ), LABEL_RIGHT, 0 ), 2, 0, AContainer::CO_FIXNR + AContainer::ACONT_CENTERV );
+    ac1_globsets->addWidget( new Text( aguix, 0, 0, catalog.getLocale( 1560 ) ), 3, 0, AContainer::CO_FIXNR + AContainer::ACONT_CENTERV );
+    m_exclude_label_sg = ac1_globsets->addWidget( new StringGadget( aguix, 0, 0, 100, "exclude_search", 0 ), 4, 0, AContainer::CO_INCW );
+
     m_newsearchb = (Button*)ac1_globsets->add( new Button( aguix,
                                                            0,
                                                            0,
                                                            catalog.getLocale( 728 ),
-                                                           0 ), 4, 0, AContainer::CO_FIX );
+                                                           0 ), 6, 0, AContainer::CO_FIX );
 
     AContainer *ac1_set2 = acsw1_1->add( new AContainer( subwin1, 2, 1 ), 0, 2 );
     ac1_set2->setMinSpace( 5 );
@@ -1007,7 +1128,7 @@ void SearchOpBG::createWindow()
     acsub3->setMaxSpace( 5 );
     acsub3->setBorderWidth( 0 );
 
-    acsub3->add( new Text( aguix, 0, 0, catalog.getLocale( 940 ) ), 0, 0, AContainer::CO_FIX );
+    acsub3->add( new Text( aguix, 0, 0, catalog.getLocale( 940 ) ), 0, 0, AContainer::CO_FIXNR + AContainer::ACONT_CENTER );
     m_max_vfs_depth_sg = (StringGadget*)acsub3->add( new StringGadget( aguix, 0, 0, aguix->getTextWidth( "MMMMMMMMM" ), "3", 0 ),
                                                      1, 0, AContainer::CO_FIX );
 
@@ -1057,7 +1178,9 @@ void SearchOpBG::createWindow()
 
     m_viewb = (Button*)ac1_4_1->add( new Button( aguix, 0, 0, catalog.getLocale( 731 ), 0 ), 0, 0, AContainer::CO_INCW );
     m_editb = (Button*)ac1_4_1->add( new Button( aguix, 0, 0, catalog.getLocale( 732 ), 0 ), 1, 0, AContainer::CO_INCW );
-    m_removeb = (Button*)ac1_4_1->add( new Button( aguix, 0, 0, catalog.getLocale( 733 ), 0 ),
+    m_removeb = (Button*)ac1_4_1->add( new Button( aguix, 0, 0,
+                                                   catalog.getLocale( 733 ),
+                                                   catalog.getLocale( 1555 ), 0 ),
                                      2, 0, AContainer::CO_INCW );
   
     AContainerBB *ac1_res_store = dynamic_cast<AContainerBB*>( ac1->add( new AContainerBB( m_win, 1, 3 ), 0, 2 ) );
@@ -1069,12 +1192,15 @@ void SearchOpBG::createWindow()
     _storelv = (FieldListView*)ac1_res_store->add( new FieldListView( aguix, 0, 0, 100, 5 * aguix->getCharHeight(), 0 ), 0, 1, AContainer::CO_MIN );
     _storelv->setHBarState( 0 );
     _storelv->setVBarState( 2 );
-    _storelv->setNrOfFields( 5 );
+    _storelv->setNrOfFields( 8 );
     _storelv->setFieldText( 0, catalog.getLocale( 963 ) );
     _storelv->setFieldText( 1, catalog.getLocale( 735 ) );
-    _storelv->setFieldText( 2, catalog.getLocale( 729 ) );
-    _storelv->setFieldText( 3, catalog.getLocale( 736 ) );
-    _storelv->setFieldText( 4, catalog.getLocale( 737 ) );
+    _storelv->setFieldText( 2, catalog.getLocale( 1562 ) );
+    _storelv->setFieldText( 3, catalog.getLocale( 1563 ) );
+    _storelv->setFieldText( 4, catalog.getLocale( 729 ) );
+    _storelv->setFieldText( 5, catalog.getLocale( 736 ) );
+    _storelv->setFieldText( 6, catalog.getLocale( 1567 ) );
+    _storelv->setFieldText( 7, catalog.getLocale( 737 ) );
     _storelv->setConsiderHeaderWidthForDynamicWidth( true );
     _storelv->setShowHeader( true );
     _storelv->setAcceptFocus( true );
@@ -1123,6 +1249,8 @@ void SearchOpBG::createWindow()
     m_startb->resize( startb_maxwidth, m_startb->getHeight() );
     ac1_5->readLimits();
 
+    updateExpressionHelp();
+
     m_win->setDoTabCycling( true );
     m_win->contMaximize( true );
 
@@ -1168,6 +1296,7 @@ void SearchOpBG::createWindow()
     _startdir_sg->selectAll( false );
     _matchname_sg->selectAll( false );
     _matchcontent_sg->selectAll( false );
+    m_match_expression_sg->selectAll( false );
 
     m_win->show();
     m_options_k1->show();
@@ -1222,7 +1351,11 @@ void SearchOpBG::handleAGUIXMessage( AGMessage &msg )
                 clearStoredResults();
                 newSearch();
             } else if ( msg.button.button == m_removeb ) {
-                removeSelectedResults();
+                if ( msg.button.state == 2 ) {
+                    removeByBaseDir();
+                } else {
+                    removeSelectedResults();
+                }
             } else if ( msg.button.button == m_viewb ) {
                 viewResult();
             } else if ( msg.button.button == m_editb ) {
@@ -1245,6 +1378,26 @@ void SearchOpBG::handleAGUIXMessage( AGMessage &msg )
                 Requester req( aguix, m_win );
                 req.request( catalog.getLocale( 124 ), catalog.getLocale( 820 ), catalog.getLocale( 11 ) );
             }
+
+            for ( size_t pos = 0; pos < m_expression_help_buttons.size(); pos++ ) {
+                if ( msg.button.button == m_expression_help_buttons[pos] ) {
+                    std::string new_expr = m_match_expression_sg->getText();
+
+                    if ( ! m_expression_help_token[pos].empty() ) {
+                        if ( ! new_expr.empty() &&
+                             ! AGUIXUtils::starts_with( m_expression_help_token[pos], "," ) ) {
+                            new_expr += " ";
+                        }
+                        new_expr += m_expression_help_token[pos];
+                    } else {
+                        new_expr += "<value>";
+                    }
+
+                    m_match_expression_sg->setText( new_expr.c_str() );
+                    updateExpressionHelp();
+                    break;
+                }
+            }
             break;
         case AG_FIELDLV_ONESELECT:
         case AG_FIELDLV_MULTISELECT:
@@ -1281,6 +1434,12 @@ void SearchOpBG::handleAGUIXMessage( AGMessage &msg )
                     _matchcontent_sg->takeFocus();
                 }
             } else if ( msg.stringgadget.sg == _matchcontent_sg ) {
+                if ( msg.stringgadget.keystate == ControlMask ) {
+                    startSearch();
+                } else {
+                    m_match_expression_sg->takeFocus();
+                }
+            } else if ( msg.stringgadget.sg == m_match_expression_sg ) {
                 startSearch();
             } else if ( msg.stringgadget.sg == m_younger_sg ) {
                 m_younger_cb->setState( true );
@@ -1292,6 +1451,11 @@ void SearchOpBG::handleAGUIXMessage( AGMessage &msg )
                 m_size_leq_cb->setState( true );
             }
             break;
+        case AG_STRINGGADGET_CONTENTCHANGE:
+            if ( msg.stringgadget.sg == m_match_expression_sg ) {
+                updateExpressionHelp();
+            }
+            break;
         case AG_KEYPRESSED:
             if ( m_win->isParent( msg.key.window, false ) ) {
                 if ( msg.key.key == XK_Return && _resultlv->getHasFocus() == true ) {
@@ -1457,3 +1621,47 @@ void SearchOpBG::updateWindowTitle()
 
     m_win->setTitle( new_title );
 }
+
+void SearchOpBG::updateExpressionHelp()
+{
+    ExprFilter expr( m_match_expression_sg->getText(), { .disabled_tokens = { "type", "state" } } );
+    size_t pos = 0;
+
+    for ( auto &e : expr.getListOfExpectedTokens() ) {
+        if ( pos >= 15 ) break;
+        if ( pos >= m_expression_help_buttons.size() ) {
+            Button *b = m_expression_help_ac->addWidget( new Button( Worker::getAGUIX(),
+                                                                     0,
+                                                                     0,
+                                                                     e.description.c_str(),
+                                                                     0 ), pos, 0, AContainer::CO_FIX );
+            b->show();
+            m_expression_help_buttons.push_back( b );
+            m_expression_help_token.push_back( e.token );
+        } else {
+            m_expression_help_buttons[pos]->show();
+            m_expression_help_buttons[pos]->setText( 0, e.description.c_str() );
+            m_expression_help_buttons[pos]->resize( m_expression_help_buttons[pos]->getMaximumWidth(),
+                                                    m_expression_help_buttons[pos]->getHeight() );
+            m_expression_help_token[pos] = e.token;
+        }
+        pos++;
+    }
+
+    while ( pos < m_expression_help_buttons.size() ) {
+        m_expression_help_buttons[pos]->hide();
+        pos++;
+    }
+
+    if ( strlen( m_match_expression_sg->getText() ) == 0 || expr.valid() ) {
+        m_expression_valid_t->setText( catalog.getLocale( 1568 ) );
+    } else {
+        m_expression_valid_t->setText( catalog.getLocale( 1569 ) );
+    }
+
+    m_expression_help_ac->readLimits();
+    m_expression_ac->readLimits();
+    _start_ac->resize( _start_ac->getWidth(),
+                       _start_ac->getHeight() );
+    _start_ac->rearrange();
+}
