@@ -281,6 +281,8 @@ StartProgOp::startprog( std::shared_ptr< WPUContext > wpu, ActionMessage *am )
               m1->_start == STARTPROGOP_SHOW_OUTPUT_CUSTOM_ATTRIBUTE ) {
       auto vdm = dynamic_cast< VirtualDirMode * >( lm1 );
       if ( vdm ) {
+          bool prev_use_virtual_copies = wpu->getUseVirtualTempCopies();
+          wpu->setUseVirtualTempCopies( true );
           vdm->processCustomAttribute( am,
                                        { .command_str = m1->_command,
                                          .dont_cd = dontcd,
@@ -289,7 +291,7 @@ StartProgOp::startprog( std::shared_ptr< WPUContext > wpu, ActionMessage *am )
                                        },
                                        tfe,
                                        tfe_id );
-          
+          wpu->setUseVirtualTempCopies( prev_use_virtual_copies );
       }
   } else if ( m1->_startmode == StartSettings::STARTPROG ) {
     tmpname = Datei::createTMPName();
