@@ -1,6 +1,6 @@
 /* togglelistermodeop.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2001-2014 Ralf Hoffmann.
+ * Copyright (C) 2001-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -64,7 +64,7 @@ ToggleListermodeOp::getName()
 }
 
 int
-ToggleListermodeOp::run( WPUContext *wpu, ActionMessage *msg )
+ToggleListermodeOp::run( std::shared_ptr< WPUContext > wpu, ActionMessage *msg )
 {
   int actmode;
   ListerMode *lm1;
@@ -172,6 +172,15 @@ ToggleListermodeOp::configure()
           if(msg->button.button==okb) endmode=0;
           else if(msg->button.button==cb) endmode=1;
           break;
+        case AG_KEYPRESSED:
+            if ( win->isParent( msg->key.window, false ) == true ) {
+                switch ( msg->key.key ) {
+                    case XK_Escape:
+                        endmode = 1;
+                        break;
+                }
+            }
+            break;
       }
       aguix->ReplyMessage(msg);
     }
