@@ -76,6 +76,9 @@ static void updateSortTabTitle( Kartei *k,
         case 10:
             new_text += "E";
             break;
+        case 11:
+            new_text += "X";
+            break;
         default:
             new_text += "N";
             break;
@@ -216,6 +219,7 @@ int VirtualDirMode::configure()
     sort_cyb->addOption( std::string( catalog.getLocale( 553 ) ) + " (L)" );
     sort_cyb->addOption( std::string( catalog.getLocale( 907 ) ) + " (P)" );
     sort_cyb->addOption( std::string( catalog.getLocale( 1240 ) ) + " (E)" );
+    sort_cyb->addOption( std::string( catalog.getLocale( 1534 ) ) + " (X)" );
     sort_cyb->resize( sort_cyb->getMaxSize(),
                       sort_cyb->getHeight() );
 
@@ -229,6 +233,7 @@ int VirtualDirMode::configure()
     else if ( ( sortmode & 0xff ) == SORT_NLINK ) sort_cyb->setOption( 8 );
     else if ( ( sortmode & 0xff ) == SORT_PERMISSION ) sort_cyb->setOption( 9 );
     else if ( ( sortmode & 0xff ) == SORT_EXTENSION ) sort_cyb->setOption( 10 );
+    else if ( ( sortmode & 0xff ) == SORT_CUSTOM_ATTRIBUTE ) sort_cyb->setOption( 11 );
     else sort_cyb->setOption( 0 );
 
     auto reverse_sort_chb = sw11_ac1->addWidget( new ChooseButton( aguix, 0, 0,
@@ -764,6 +769,9 @@ int VirtualDirMode::configure()
             case 10:
                 tsortmode = SORT_EXTENSION;
                 break;
+            case 11:
+                tsortmode = SORT_CUSTOM_ATTRIBUTE;
+                break;
             default:
                 tsortmode = SORT_NAME;
                 break;
