@@ -1,6 +1,6 @@
 /* wconfig_generalconf.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2007-2015 Ralf Hoffmann.
+ * Copyright (C) 2007-2021 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -26,6 +26,7 @@
 #include "worker_locale.h"
 #include "aguix/choosebutton.h"
 #include "aguix/cyclebutton.h"
+#include "aguix/stringgadget.h"
 
 GeneralConfPanel::GeneralConfPanel( AWindow &basewin, WConfig &baseconfig ) : WConfigPanel( basewin, baseconfig )
 {
@@ -46,9 +47,9 @@ int GeneralConfPanel::create()
     ac1->add( new Text( _aguix, 0, 0, catalog.getLocale( 784 ) ), 0, 0, AContainer::CO_INCWNR );
 
 #if 0
-    AContainerBB *ac1_1 = (AContainerBB*)ac1->add( new AContainerBB( this, 1, 4 ), 0, 1 );
+    AContainerBB *ac1_1 = (AContainerBB*)ac1->add( new AContainerBB( this, 1, 5 ), 0, 1 );
 #else
-    AContainerBB *ac1_1 = (AContainerBB*)ac1->add( new AContainerBB( this, 1, 3 ), 0, 1 );
+    AContainerBB *ac1_1 = (AContainerBB*)ac1->add( new AContainerBB( this, 1, 4 ), 0, 1 );
 #endif
     ac1_1->setMinSpace( 5 );
     ac1_1->setMaxSpace( 5 );
@@ -94,6 +95,14 @@ int GeneralConfPanel::create()
                                                                 LABEL_LEFT,
                                                                 0 ), 0, 2, AContainer::CO_INCWNR );
 
+    AContainer *ac1_1_2 = ac1_1->add( new AContainer( this, 2, 1 ), 0, 3 );
+    ac1_1_2->setBorderWidth( 0 );
+    ac1_1_2->setMinSpace( 5 );
+    ac1_1_2->setMaxSpace( 5 );
+
+    ac1_1_2->add( new Text( _aguix, 0, 0, catalog.getLocale(1385) ), 0, 0, AContainer::CO_FIX );
+    m_bg_check_sg = ac1_1_2->addWidget( new StringGadget( _aguix, 0, 0, 200, _baseconfig.getDisableBGCheckPrefix().c_str(), 0 ), 1, 0, AContainer::CO_INCW );
+
 #if 0
     m_dialog_cb = (ChooseButton*)ac1_1->add( new ChooseButton( _aguix,
                                                                0,
@@ -121,6 +130,8 @@ int GeneralConfPanel::saveValues()
         _baseconfig.setUseStringCompareMode( StringComparator::STRING_COMPARE_REGULAR );
     }
 
+    _baseconfig.setDisableBGCheckPrefix( m_bg_check_sg->getText() );
+
 #if 0
     _baseconfig.setApplyWindowDialogType( m_dialog_cb->getState() );
 #endif
