@@ -292,7 +292,7 @@ int LayoutPanel::saveValues()
     return 0;
 }
 
-void LayoutPanel::run( Widget *elem, AGMessage &msg )
+void LayoutPanel::run( Widget *elem, const AGMessage &msg )
 {
     if ( msg.type == AG_BUTTONCLICKED ) {
         if ( msg.button.button == m_add_element_b ) {
