@@ -288,7 +288,7 @@ int ListerPanel::saveValues()
   return 0;
 }
 
-void ListerPanel::run( Widget *elem, AGMessage &msg )
+void ListerPanel::run( Widget *elem, const AGMessage &msg )
 {
   char tstr[ A_BYTESFORNUMBER( int ) ];
 
