@@ -1,6 +1,6 @@
 /* wconfig_mouse.cc
  * This file belongs to Worker, a file manager for UN*X/X11.
- * Copyright (C) 2006-2007,2011 Ralf Hoffmann.
+ * Copyright (C) 2006-2025 Ralf Hoffmann.
  * You can contact me at: ralf@boomerangsworld.de
  *   or http://www.boomerangsworld.de/worker
  *
@@ -24,6 +24,7 @@
 #include "worker.h"
 #include "aguix/acontainerbb.h"
 #include "aguix/cyclebutton.h"
+#include "aguix/choosebutton.h"
 #include "worker_locale.h"
 
 MousePanel::MousePanel( AWindow &basewin, WConfig &baseconfig ) : WConfigPanel( basewin, baseconfig )
@@ -38,7 +39,7 @@ int MousePanel::create()
 {
   Panel::create();
 
-  AContainer *ac1 = setContainer( new AContainer( this, 1, 3 ), true );
+  AContainer *ac1 = setContainer( new AContainer( this, 1, 4 ), true );
   ac1->setMinSpace( 5 );
   ac1->setMaxSpace( 5 );
 
@@ -287,6 +288,13 @@ int MousePanel::create()
     cycbsmb->setOption( 0 );
   }
 
+  m_delayed_content_menu_cb = ac1->addWidget( new ChooseButton( _aguix,
+                                                                0, 0,
+                                                                _baseconfig.getMouseContextMenuWithDelayedActivate(),
+                                                                catalog.getLocale( 1572 ),
+                                                                LABEL_LEFT, 0 ),
+                                              0, 3, AContainer::CO_FIXNR );
+
   contMaximize( true );
   return 0;
 }
@@ -347,6 +355,9 @@ int MousePanel::saveValues()
       _baseconfig.setMouseContextMod( 0 );
       break;
   }
+
+  _baseconfig.setMouseContextMenuWithDelayedActivate( m_delayed_content_menu_cb->getState() );
+
   return 0;
 }
 
