@@ -193,6 +193,11 @@ WPUContext::WPUContext( WPUContext &other,
                 msg )
 {
     m_stacks = other.getStacks();
+
+    m_recursive = other.m_recursive;
+    m_take_dirs = other.m_take_dirs;
+    m_use_virtual_temp_copies = other.m_use_virtual_temp_copies;
+    m_use_extended_basename = other.m_use_extended_basename;
 }
 
 
